@@ -26,6 +26,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "../../src/client/datadevicemanager.h"
 #include "../../src/client/datasource.h"
 #include "../../src/client/compositor.h"
+#include "../../src/client/keyboard.h"
 #include "../../src/client/pointer.h"
 #include "../../src/client/registry.h"
 #include "../../src/client/seat.h"
@@ -51,6 +52,8 @@ private Q_SLOTS:
     void testDrag();
     void testDragInternally();
     void testSetSelection();
+    void testSendSelectionOnSeat();
+    void testReplaceSource();
     void testDestroy();
 
 private:
@@ -191,6 +194,17 @@ void TestDataDevice::testCreate()
     QVERIFY(!deviceInterface->icon());
     QVERIFY(!deviceInterface->selection());
     QVERIFY(deviceInterface->parentResource());
+
+    QVERIFY(!m_seatInterface->selection());
+    m_seatInterface->setSelection(deviceInterface);
+    QCOMPARE(m_seatInterface->selection(), deviceInterface);
+
+    // and destroy
+    QSignalSpy destroyedSpy(deviceInterface, &QObject::destroyed);
+    QVERIFY(destroyedSpy.isValid());
+    dataDevice.reset();
+    QVERIFY(destroyedSpy.wait());
+    QVERIFY(!m_seatInterface->selection());
 }
 
 void TestDataDevice::testDrag()
@@ -372,6 +386,147 @@ void TestDataDevice::testSetSelection()
     QCOMPARE(selectionChangedSpy.count(), 1);
     QCOMPARE(selectionClearedSpy.count(), 1);
     QVERIFY(!deviceInterface->selection());
+
+    // set another selection
+    dataDevice->setSelection(2, dataSource.data());
+    QVERIFY(selectionChangedSpy.wait());
+    // now unbind the dataDevice
+    QSignalSpy unboundSpy(deviceInterface, &DataDeviceInterface::unbound);
+    QVERIFY(unboundSpy.isValid());
+    dataDevice.reset();
+    QVERIFY(unboundSpy.wait());
+    // send a selection to the unbound data device
+    deviceInterface->sendSelection(deviceInterface);
+}
+
+void TestDataDevice::testSendSelectionOnSeat()
+{
+    // this test verifies that the selection is sent when setting a focused keyboard
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    // first add keyboard support to Seat
+    QSignalSpy keyboardChangedSpy(m_seat, &Seat::hasKeyboardChanged);
+    QVERIFY(keyboardChangedSpy.isValid());
+    m_seatInterface->setHasKeyboard(true);
+    QVERIFY(keyboardChangedSpy.wait());
+    // now create DataDevice, Keyboard and a Surface
+    QSignalSpy dataDeviceCreatedSpy(m_dataDeviceManagerInterface, &DataDeviceManagerInterface::dataDeviceCreated);
+    QVERIFY(dataDeviceCreatedSpy.isValid());
+    QScopedPointer<DataDevice> dataDevice(m_dataDeviceManager->getDataDevice(m_seat));
+    QVERIFY(dataDevice->isValid());
+    QVERIFY(dataDeviceCreatedSpy.wait());
+    auto serverDataDevice = dataDeviceCreatedSpy.first().first().value<DataDeviceInterface*>();
+    QVERIFY(serverDataDevice);
+    QScopedPointer<Keyboard> keyboard(m_seat->createKeyboard());
+    QVERIFY(keyboard->isValid());
+    QSignalSpy surfaceCreatedSpy(m_compositorInterface, &CompositorInterface::surfaceCreated);
+    QVERIFY(surfaceCreatedSpy.isValid());
+    QScopedPointer<Surface> surface(m_compositor->createSurface());
+    QVERIFY(surface->isValid());
+    QVERIFY(surfaceCreatedSpy.wait());
+
+    auto serverSurface = surfaceCreatedSpy.first().first().value<SurfaceInterface*>();
+    QVERIFY(serverSurface);
+    m_seatInterface->setFocusedKeyboardSurface(serverSurface);
+
+    // now set the selection
+    QScopedPointer<DataSource> dataSource(m_dataDeviceManager->createDataSource());
+    QVERIFY(dataSource->isValid());
+    dataSource->offer(QStringLiteral("text/plain"));
+    dataDevice->setSelection(1, dataSource.data());
+    // we should get a selection offered for that on the data device
+    QSignalSpy selectionOfferedSpy(dataDevice.data(), &DataDevice::selectionOffered);
+    QVERIFY(selectionOfferedSpy.isValid());
+    QVERIFY(selectionOfferedSpy.wait());
+    QCOMPARE(selectionOfferedSpy.count(), 1);
+
+    // now unfocus the keyboard
+    m_seatInterface->setFocusedKeyboardSurface(nullptr);
+    // if setting the same surface again, we should get another offer
+    m_seatInterface->setFocusedKeyboardSurface(serverSurface);
+    QVERIFY(selectionOfferedSpy.wait());
+    QCOMPARE(selectionOfferedSpy.count(), 2);
+
+    // now let's try to destroy the data device and set a focused keyboard just while the data device is being destroyedd
+    m_seatInterface->setFocusedKeyboardSurface(nullptr);
+    QSignalSpy unboundSpy(serverDataDevice, &Resource::unbound);
+    QVERIFY(unboundSpy.isValid());
+    dataDevice.reset();
+    QVERIFY(unboundSpy.wait());
+    m_seatInterface->setFocusedKeyboardSurface(serverSurface);
+}
+
+void TestDataDevice::testReplaceSource()
+{
+    // this test verifies that replacing a data source cancels the previous source
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    // first add keyboard support to Seat
+    QSignalSpy keyboardChangedSpy(m_seat, &Seat::hasKeyboardChanged);
+    QVERIFY(keyboardChangedSpy.isValid());
+    m_seatInterface->setHasKeyboard(true);
+    QVERIFY(keyboardChangedSpy.wait());
+    // now create DataDevice, Keyboard and a Surface
+    QSignalSpy dataDeviceCreatedSpy(m_dataDeviceManagerInterface, &DataDeviceManagerInterface::dataDeviceCreated);
+    QVERIFY(dataDeviceCreatedSpy.isValid());
+    QScopedPointer<DataDevice> dataDevice(m_dataDeviceManager->getDataDevice(m_seat));
+    QVERIFY(dataDevice->isValid());
+    QVERIFY(dataDeviceCreatedSpy.wait());
+    auto serverDataDevice = dataDeviceCreatedSpy.first().first().value<DataDeviceInterface*>();
+    QVERIFY(serverDataDevice);
+    QScopedPointer<Keyboard> keyboard(m_seat->createKeyboard());
+    QVERIFY(keyboard->isValid());
+    QSignalSpy surfaceCreatedSpy(m_compositorInterface, &CompositorInterface::surfaceCreated);
+    QVERIFY(surfaceCreatedSpy.isValid());
+    QScopedPointer<Surface> surface(m_compositor->createSurface());
+    QVERIFY(surface->isValid());
+    QVERIFY(surfaceCreatedSpy.wait());
+
+    auto serverSurface = surfaceCreatedSpy.first().first().value<SurfaceInterface*>();
+    QVERIFY(serverSurface);
+    m_seatInterface->setFocusedKeyboardSurface(serverSurface);
+
+    // now set the selection
+    QScopedPointer<DataSource> dataSource(m_dataDeviceManager->createDataSource());
+    QVERIFY(dataSource->isValid());
+    dataSource->offer(QStringLiteral("text/plain"));
+    dataDevice->setSelection(1, dataSource.data());
+    QSignalSpy sourceCancelledSpy(dataSource.data(), &DataSource::cancelled);
+    QVERIFY(sourceCancelledSpy.isValid());
+    // we should get a selection offered for that on the data device
+    QSignalSpy selectionOfferedSpy(dataDevice.data(), &DataDevice::selectionOffered);
+    QVERIFY(selectionOfferedSpy.isValid());
+    QVERIFY(selectionOfferedSpy.wait());
+    QCOMPARE(selectionOfferedSpy.count(), 1);
+
+    // create a second data source and replace previous one
+    QScopedPointer<DataSource> dataSource2(m_dataDeviceManager->createDataSource());
+    QVERIFY(dataSource2->isValid());
+    dataSource2->offer(QStringLiteral("text/plain"));
+    QSignalSpy sourceCancelled2Spy(dataSource2.data(), &DataSource::cancelled);
+    QVERIFY(sourceCancelled2Spy.isValid());
+    dataDevice->setSelection(1, dataSource2.data());
+    QCOMPARE(selectionOfferedSpy.count(), 1);
+    QVERIFY(sourceCancelledSpy.wait());
+    QCOMPARE(selectionOfferedSpy.count(), 2);
+    QVERIFY(sourceCancelled2Spy.isEmpty());
+
+    // create a new DataDevice and replace previous one
+    QScopedPointer<DataDevice> dataDevice2(m_dataDeviceManager->getDataDevice(m_seat));
+    QVERIFY(dataDevice2->isValid());
+    QScopedPointer<DataSource> dataSource3(m_dataDeviceManager->createDataSource());
+    QVERIFY(dataSource3->isValid());
+    dataSource3->offer(QStringLiteral("text/plain"));
+    dataDevice2->setSelection(1, dataSource3.data());
+    QVERIFY(sourceCancelled2Spy.wait());
+
+    // try to crash by first destroying dataSource3 and setting a new DataSource
+    QScopedPointer<DataSource> dataSource4(m_dataDeviceManager->createDataSource());
+    QVERIFY(dataSource4->isValid());
+    dataSource4->offer(QStringLiteral("text/plain"));
+    dataSource3.reset();
+    dataDevice2->setSelection(1, dataSource4.data());
+    QVERIFY(selectionOfferedSpy.wait());
 }
 
 void TestDataDevice::testDestroy()
