@@ -35,6 +35,8 @@ namespace KWayland
 namespace Client
 {
 
+class EventQueue;
+
 /**
  * @short Wrapper for the wl_output interface.
  *
@@ -187,6 +189,21 @@ public:
      **/
     QList<Mode> modes() const;
 
+    /**
+     * Sets the @p queue to use for bound proxies.
+     **/
+    void setEventQueue(EventQueue *queue);
+    /**
+     * @returns The event queue to use for bound proxies.
+     **/
+    EventQueue *eventQueue() const;
+
+    /**
+     * @returns The Output for the @p native wl_output. @c null if there is no Output for it.
+     * @since 5.27
+     **/
+    static Output *get(wl_output *native);
+
 Q_SIGNALS:
     /**
      * Emitted whenever at least one of the data changed.
@@ -207,6 +224,16 @@ Q_SIGNALS:
      **/
     void modeChanged(const KWayland::Client::Output::Mode &mode);
 
+    /**
+     * The corresponding global for this interface on the Registry got removed.
+     *
+     * This signal gets only emitted if the Compositor got created by
+     * Registry::createOutput
+     *
+     * @since 5.5
+     **/
+    void removed();
+
 private:
     class Private;
     QScopedPointer<Private> d;
@@ -215,6 +242,7 @@ private:
 }
 }
 
+Q_DECLARE_METATYPE(KWayland::Client::Output*)
 Q_DECLARE_METATYPE(KWayland::Client::Output::SubPixel)
 Q_DECLARE_METATYPE(KWayland::Client::Output::Transform)
 Q_DECLARE_METATYPE(KWayland::Client::Output::Mode)
