@@ -34,6 +34,8 @@ struct wl_seat;
 struct wl_shell;
 struct wl_shm;
 struct wl_subcompositor;
+struct wl_text_input_manager;
+struct zwp_text_input_manager_v2;
 struct _wl_fullscreen_shell;
 struct org_kde_kwin_outputmanagement;
 struct org_kde_kwin_outputdevice;
@@ -46,6 +48,9 @@ struct org_kde_kwin_contrast_manager;
 struct org_kde_kwin_slide_manager;
 struct org_kde_plasma_shell;
 struct org_kde_plasma_window_management;
+struct org_kde_kwin_server_decoration_manager;
+struct xdg_shell;
+struct zwp_relative_pointer_manager_v1;
 
 namespace KWayland
 {
@@ -72,7 +77,13 @@ class ContrastManager;
 class SlideManager;
 class Shell;
 class ShmPool;
+class ServerSideDecorationManager;
 class SubCompositor;
+class TextInputManager;
+class TextInputManagerUnstableV0;
+class TextInputManagerUnstableV2;
+class XdgShell;
+class RelativePointerManager;
 
 /**
  * @short Wrapper for the wl_registry interface.
@@ -128,7 +139,12 @@ public:
         Slide, ///< refers to org_kde_kwin_slide_manager
         Dpms, ///< Refers to org_kde_kwin_dpms_manager interface
         OutputManagement, ///< Refers to the wl_data_device_manager interface
-        OutputDevice     ///< Refers to the org_kde_kwin_outputdevice interface
+        OutputDevice,     ///< Refers to the org_kde_kwin_outputdevice interface
+        ServerSideDecorationManager, ///< Refers to org_kde_kwin_server_decoration_manager
+        TextInputManagerUnstableV0, ///< Refers to wl_text_input_manager, @since 5.23
+        TextInputManagerUnstableV2, ///< Refers to zwp_text_input_manager_v2, @since 5.23
+        XdgShellUnstableV5, ///< Refers to xdg_shell (unstable version 5), @since 5.25
+        RelativePointerManagerUnstableV1 ///< Refers to zwp_relative_pointer_manager_v1, @since 5.28
     };
     explicit Registry(QObject *parent = nullptr);
     virtual ~Registry();
@@ -423,6 +439,56 @@ public:
      * @since 5.5
      **/
     org_kde_kwin_dpms_manager *bindDpmsManager(uint32_t name, uint32_t version) const;
+    /**
+     * Binds the org_kde_kwin_server_decoration_manager with @p name and @p version.
+     * If the @p name does not exist or is not for the server side decoration manager interface,
+     * @c null will be returned.
+     *
+     * Prefer using createServerSideDecorationManager instead.
+     * @see createServerSideDecorationManager
+     * @since 5.6
+     **/
+    org_kde_kwin_server_decoration_manager *bindServerSideDecorationManager(uint32_t name, uint32_t version) const;
+    /**
+     * Binds the wl_text_input_manager with @p name and @p version.
+     * If the @p name does not exist or is not for the text input interface in unstable version 0,
+     * @c null will be returned.
+     *
+     * Prefer using createTextInputManager instead.
+     * @see createTextInputManager
+     * @since 5.23
+     **/
+    wl_text_input_manager *bindTextInputManagerUnstableV0(uint32_t name, uint32_t version) const;
+    /**
+     * Binds the zwp_text_input_manager_v2 with @p name and @p version.
+     * If the @p name does not exist or is not for the text input interface in unstable version 2,
+     * @c null will be returned.
+     *
+     * Prefer using createTextInputManager instead.
+     * @see createTextInputManager
+     * @since 5.23
+     **/
+    zwp_text_input_manager_v2 *bindTextInputManagerUnstableV2(uint32_t name, uint32_t version) const;
+    /**
+     * Binds the xdg_shell (unstable version 5) with @p name and @p version.
+     * If the @p name does not exist or is not for the xdg shell interface in unstable version 5,
+     * @c null will be returned.
+     *
+     * Prefer using createXdgShell instead.
+     * @see createXdgShell
+     * @since 5.25
+     **/
+    xdg_shell *bindXdgShellUnstableV5(uint32_t name, uint32_t version) const;
+    /**
+     * Binds the zwp_relative_pointer_manager_v1 with @p name and @p version.
+     * If the @p name does not exist or is not for the relative pointer interface in unstable version 1,
+     * @c null will be returned.
+     *
+     * Prefer using createRelativePointerManager instead.
+     * @see createRelativePointerManager
+     * @since 5.28
+     **/
+    zwp_relative_pointer_manager_v1 *bindRelativePointerManagerUnstableV1(uint32_t name, uint32_t version) const;
     ///@}
 
     /**
@@ -726,6 +792,77 @@ public:
      * @since 5.5
      **/
     DpmsManager *createDpmsManager(quint32 name, quint32 version, QObject *parent = nullptr);
+    /**
+     * Creates a ServerSideDecorationManager and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * Note: in case @p name is invalid or isn't for the org_kde_kwin_server_decoration_manager interface,
+     * the returned ServerSideDecorationManager will not be valid. Therefore it's recommended to call
+     * isValid on the created instance.
+     *
+     * @param name The name of the org_kde_kwin_server_decoration_manager interface to bind
+     * @param version The version or the org_kde_kwin_server_decoration_manager interface to use
+     * @param parent The parent for ServerSideDecorationManager
+     *
+     * @returns The created ServerSideDecorationManager.
+     * @since 5.6
+     **/
+    ServerSideDecorationManager *createServerSideDecorationManager(quint32 name, quint32 version, QObject *parent = nullptr);
+    /**
+     * Creates a TextInputManager and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * This factory method supports the following interfaces:
+     * @li wl_text_input_manager
+     * @li zwp_text_input_manager_v2
+     *
+     * If @p name is for one of the supported interfaces the corresponding manager will be created,
+     * otherwise @c null will be returned.
+     *
+     * @param name The name of the interface to bind
+     * @param version The version of the interface to use
+     * @param parent The parent for the TextInputManager
+     *
+     * @returns The created TextInputManager
+     * @since 5.23
+     **/
+    TextInputManager *createTextInputManager(quint32 name, quint32 version, QObject *parent = nullptr);
+    /**
+     * Creates an XdgShell and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * This factory method supports the following interfaces:
+     * @li xdg_shell (Unstable version 5)
+     *
+     * If @p name is for one of the supported interfaces the corresponding shell will be created,
+     * otherwise @c null will be returned.
+     *
+     * @param name The name of the interface to bind
+     * @param version The version of the interface to use
+     * @param parent The parent for the XdgShell
+     *
+     * @returns The created XdgShell
+     * @since 5.25
+     **/
+    XdgShell *createXdgShell(quint32 name, quint32 version, QObject *parent = nullptr);
+    /**
+     * Creates a RelativePointerManager and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * This factory method supports the following interfaces:
+     * @li zwp_relative_pointer_manager_v1
+     *
+     * If @p name is for one of the supported interfaces the corresponding manager will be created,
+     * otherwise @c null will be returned.
+     *
+     * @param name The name of the interface to bind
+     * @param version The version of the interface to use
+     * @param parent The parent for the RelativePointerManager
+     *
+     * @returns The created RelativePointerManager
+     * @since 5.28
+     **/
+    RelativePointerManager *createRelativePointerManager(quint32 name, quint32 version, QObject *parent = nullptr);
     ///@}
 
     /**
@@ -866,6 +1003,41 @@ Q_SIGNALS:
      * @since 5.5
      **/
     void dpmsAnnounced(quint32 name, quint32 version);
+    /**
+     * Emitted whenever a org_kde_kwin_server_decoration_manager interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.6
+     **/
+    void serverSideDecorationManagerAnnounced(quint32 name, quint32 version);
+    /**
+     * Emitted whenever a wl_text_input_manager interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.23
+     **/
+    void textInputManagerUnstableV0Announced(quint32 name, quint32 version);
+    /**
+     * Emitted whenever a zwp_text_input_manager_v2 interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.23
+     **/
+    void textInputManagerUnstableV2Announced(quint32 name, quint32 version);
+    /**
+     * Emitted whenever a xdg_shell (unstable version 5) interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.25
+     **/
+    void xdgShellUnstableV5Announced(quint32 name, quint32 version);
+    /**
+     * Emitted whenever a zwp_relative_pointer_manager_v1 interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.28
+     **/
+    void relativePointerManagerUnstableV1Announced(quint32 name, quint32 version);
     ///@}
     /**
      * @name Interface removed signals.
@@ -977,6 +1149,36 @@ Q_SIGNALS:
      * @since 5.5
      **/
     void dpmsRemoved(quint32 name);
+    /**
+     * Emitted whenever a org_kde_kwin_server_decoration_manager interface gets removed.
+     * @param name The name for the removed interface
+     * @since 5.6
+     **/
+    void serverSideDecorationManagerRemoved(quint32 name);
+    /**
+     * Emitted whenever a wl_text_input_manager interface gets removed.
+     * @param name The name for the removed interface
+     * @since 5.23
+     **/
+    void textInputManagerUnstableV0Removed(quint32 name);
+    /**
+     * Emitted whenever a zwp_text_input_manager_v2 interface gets removed.
+     * @param name The name for the removed interface
+     * @since 5.23
+     **/
+    void textInputManagerUnstableV2Removed(quint32 name);
+    /**
+     * Emitted whenever an xdg_shell (unstable version 5) interface gets removed.
+     * @param name The name for the removed interface
+     * @since 5.25
+     **/
+    void xdgShellUnstableV5Removed(quint32 name);
+    /**
+     * Emitted whenever a zwp_relative_pointer_manager_v1 interface gets removed.
+     * @param name The name for the removed interface
+     * @since 5.28
+     **/
+    void relativePointerManagerUnstableV1Removed(quint32 name);
     ///@}
     /**
      * Generic announced signal which gets emitted whenever an interface gets
