@@ -20,8 +20,14 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "shell.h"
 #include "event_queue.h"
 #include "output.h"
+#include "seat.h"
 #include "surface.h"
 #include "wayland_pointer_p.h"
+// Qt
+#include <QGuiApplication>
+#include <QVector>
+#include <QWindow>
+#include <qpa/qplatformnativeinterface.h>
 // Wayland
 #include <wayland-client-protocol.h>
 
@@ -126,6 +132,7 @@ public:
 
     WaylandPointer<wl_shell_surface, wl_shell_surface_destroy> surface;
     QSize size;
+    static QVector<ShellSurface*> s_surfaces;
 
 private:
     void ping(uint32_t serial);
@@ -137,6 +144,8 @@ private:
     static const struct wl_shell_surface_listener s_listener;
 };
 
+QVector<ShellSurface*> ShellSurface::Private::s_surfaces = QVector<ShellSurface*>();
+
 ShellSurface::Private::Private(ShellSurface *q)
     : q(q)
 {
@@ -150,14 +159,71 @@ void ShellSurface::Private::setup(wl_shell_surface *s)
     wl_shell_surface_add_listener(surface, &s_listener, this);
 }
 
+ShellSurface *ShellSurface::fromWindow(QWindow *window)
+{
+    if (!window) {
+        return nullptr;
+    }
+    if (!QGuiApplication::platformName().contains(QStringLiteral("wayland"), Qt::CaseInsensitive)) {
+        return nullptr;
+    }
+    QPlatformNativeInterface *native = qApp->platformNativeInterface();
+    if (!native) {
+        return nullptr;
+    }
+    window->create();
+    wl_shell_surface *s = reinterpret_cast<wl_shell_surface*>(native->nativeResourceForWindow(QByteArrayLiteral("wl_shell_surface"), window));
+    if (!s) {
+        return nullptr;
+    }
+    if (auto surface = get(s)) {
+        return surface;
+    }
+    ShellSurface *surface = new ShellSurface(window);
+    surface->d->surface.setup(s, true);
+    return surface;
+}
+
+ShellSurface *ShellSurface::fromQtWinId(WId wid)
+{
+    QWindow *window = nullptr;
+
+    for (auto win : qApp->allWindows()) {
+        if (win->winId() == wid) {
+            window = win;
+            break;
+        }
+    }
+
+    if (!window) {
+        return nullptr;
+    }
+    return fromWindow(window);
+}
+
+ShellSurface *ShellSurface::get(wl_shell_surface *native)
+{
+    auto it = std::find_if(Private::s_surfaces.constBegin(), Private::s_surfaces.constEnd(),
+        [native](ShellSurface *s) {
+            return s->d->surface == native;
+        }
+    );
+    if (it != Private::s_surfaces.constEnd()) {
+        return *(it);
+    }
+    return nullptr;
+}
+
 ShellSurface::ShellSurface(QObject *parent)
     : QObject(parent)
     , d(new Private(this))
 {
+    Private::s_surfaces << this;
 }
 
 ShellSurface::~ShellSurface()
 {
+    Private::s_surfaces.removeOne(this);
     release();
 }
 
@@ -171,11 +237,13 @@ void ShellSurface::destroy()
     d->surface.destroy();
 }
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 const struct wl_shell_surface_listener ShellSurface::Private::s_listener = {
     pingCallback,
     configureCallback,
     popupDoneCallback
 };
+#endif
 
 void ShellSurface::Private::configureCallback(void *data, wl_shell_surface *shellSurface, uint32_t edges, int32_t width, int32_t height)
 {
@@ -237,6 +305,55 @@ void ShellSurface::setToplevel()
     wl_shell_surface_set_toplevel(d->surface);
 }
 
+void ShellSurface::setTransient(Surface *parent, const QPoint &offset, TransientFlags flags)
+{
+    Q_ASSERT(isValid());
+    uint32_t wlFlags = 0;
+    if (flags.testFlag(TransientFlag::NoFocus)) {
+        wlFlags |= WL_SHELL_SURFACE_TRANSIENT_INACTIVE;
+    }
+    wl_shell_surface_set_transient(d->surface, *parent, offset.x(), offset.y(), wlFlags);
+}
+
+void ShellSurface::requestMove(Seat *seat, quint32 serial)
+{
+    Q_ASSERT(isValid());
+    Q_ASSERT(seat);
+
+    wl_shell_surface_move(d->surface, *seat, serial);
+}
+
+void ShellSurface::requestResize(Seat *seat, quint32 serial, Qt::Edges edges)
+{
+    Q_ASSERT(isValid());
+    Q_ASSERT(seat);
+
+    uint wlEdge = WL_SHELL_SURFACE_RESIZE_NONE;
+    if (edges.testFlag(Qt::TopEdge)) {
+        if (edges.testFlag(Qt::LeftEdge) && ((edges & ~Qt::LeftEdge) == Qt::TopEdge)) {
+            wlEdge = WL_SHELL_SURFACE_RESIZE_TOP_LEFT;
+        } else if (edges.testFlag(Qt::RightEdge) && ((edges & ~Qt::RightEdge) == Qt::TopEdge)) {
+            wlEdge = WL_SHELL_SURFACE_RESIZE_TOP_RIGHT;
+        } else if ((edges & ~Qt::TopEdge) == Qt::Edges()) {
+            wlEdge = WL_SHELL_SURFACE_RESIZE_TOP;
+        }
+    } else if (edges.testFlag(Qt::BottomEdge)) {
+        if (edges.testFlag(Qt::LeftEdge) && ((edges & ~Qt::LeftEdge) == Qt::BottomEdge)) {
+            wlEdge = WL_SHELL_SURFACE_RESIZE_BOTTOM_LEFT;
+        } else if (edges.testFlag(Qt::RightEdge) && ((edges & ~Qt::RightEdge) == Qt::BottomEdge)) {
+            wlEdge = WL_SHELL_SURFACE_RESIZE_BOTTOM_RIGHT;
+        } else if ((edges & ~Qt::BottomEdge) == Qt::Edges()) {
+            wlEdge = WL_SHELL_SURFACE_RESIZE_BOTTOM;
+        }
+    } else if (edges.testFlag(Qt::RightEdge) && ((edges & ~Qt::RightEdge) == Qt::Edges())) {
+        wlEdge = WL_SHELL_SURFACE_RESIZE_RIGHT;
+    } else if (edges.testFlag(Qt::LeftEdge) && ((edges & ~Qt::LeftEdge) == Qt::Edges())) {
+        wlEdge = WL_SHELL_SURFACE_RESIZE_LEFT;
+    }
+
+    wl_shell_surface_resize(d->surface, *seat, serial, wlEdge);
+}
+
 QSize ShellSurface::size() const
 {
     return d->size;
