@@ -132,14 +132,18 @@ void SeatInterface::Private::updatePointerButtonState(quint32 button, Pointer::S
     it.value() = state;
 }
 
-void SeatInterface::Private::updateKey(quint32 key, Keyboard::State state)
+bool SeatInterface::Private::updateKey(quint32 key, Keyboard::State state)
 {
     auto it = keys.states.find(key);
     if (it == keys.states.end()) {
         keys.states.insert(key, state);
-        return;
+        return true;
+    }
+    if (it.value() == state) {
+        return false;
     }
     it.value() = state;
+    return true;
 }
 
 void SeatInterface::Private::sendName(wl_resource *r)
@@ -186,16 +190,33 @@ T *interfaceForSurface(SurfaceInterface *surface, const QVector<T*> &interfaces)
     }
     return nullptr;
 }
+
+template <typename T>
+static
+QVector<T *> interfacesForSurface(SurfaceInterface *surface, const QVector<T*> &interfaces)
+{
+    QVector<T *> ret;
+    if (!surface) {
+        return ret;
+    }
+
+    for (auto it = interfaces.begin(); it != interfaces.end(); ++it) {
+        if ((*it)->client() == surface->client() && (*it)->resource()) {
+            ret << *it;
+        }
+    }
+    return ret;
+}
 }
 
-PointerInterface *SeatInterface::Private::pointerForSurface(SurfaceInterface *surface) const
+QVector<PointerInterface *> SeatInterface::Private::pointersForSurface(SurfaceInterface *surface) const
 {
-    return interfaceForSurface(surface, pointers);
+    return interfacesForSurface(surface, pointers);
 }
 
-KeyboardInterface *SeatInterface::Private::keyboardForSurface(SurfaceInterface *surface) const
+QVector<KeyboardInterface *> SeatInterface::Private::keyboardsForSurface(SurfaceInterface *surface) const
 {
-    return interfaceForSurface(surface, keyboards);
+    return interfacesForSurface(surface, keyboards);
 }
 
 TouchInterface *SeatInterface::Private::touchForSurface(SurfaceInterface *surface) const
@@ -271,7 +292,7 @@ void SeatInterface::Private::registerDataDevice(DataDeviceInterface *dataDevice)
         // same client?
         if (keys.focus.surface->client() == dataDevice->client()) {
             keys.focus.selection = dataDevice;
-            if (currentSelection) {
+            if (currentSelection && currentSelection->selection()) {
                 dataDevice->sendSelection(currentSelection);
             }
         }
@@ -318,9 +339,25 @@ void SeatInterface::Private::endDrag(quint32 serial)
     emit q->dragEnded();
 }
 
+void SeatInterface::Private::cancelPreviousSelection(DataDeviceInterface *dataDevice)
+{
+    if (!currentSelection) {
+        return;
+    }
+    if (auto s = currentSelection->selection()) {
+        if (currentSelection != dataDevice) {
+            // only if current selection is not on the same device
+            // that would cancel the newly set source
+            s->cancel();
+        }
+    }
+}
+
 void SeatInterface::Private::updateSelection(DataDeviceInterface *dataDevice, bool set)
 {
     if (keys.focus.surface && (keys.focus.surface->client() == dataDevice->client())) {
+        // cancel the previous selection
+        cancelPreviousSelection(dataDevice);
         // new selection on a data device belonging to current keyboard focus
         currentSelection = dataDevice;
     }
@@ -396,18 +433,20 @@ void SeatInterface::Private::getPointer(wl_client *client, wl_resource *resource
     pointers << pointer;
     if (globalPointer.focus.surface && globalPointer.focus.surface->client() == clientConnection) {
         // this is a pointer for the currently focused pointer surface
-        if (!globalPointer.focus.pointer) {
-            globalPointer.focus.pointer = pointer;
-            pointer->setFocusedSurface(globalPointer.focus.surface, globalPointer.focus.serial);
+        globalPointer.focus.pointers << pointer;
+        pointer->setFocusedSurface(globalPointer.focus.surface, globalPointer.focus.serial);
+        if (globalPointer.focus.pointers.count() == 1) {
+            // got a new pointer
             emit q->focusedPointerChanged(pointer);
         }
     }
     QObject::connect(pointer, &QObject::destroyed, q,
         [pointer,this] {
             pointers.removeAt(pointers.indexOf(pointer));
-            if (globalPointer.focus.pointer == pointer) {
-                globalPointer.focus.pointer = nullptr;
-                emit q->focusedPointerChanged(nullptr);
+            if (globalPointer.focus.pointers.removeOne(pointer)) {
+                if (globalPointer.focus.pointers.isEmpty()) {
+                    emit q->focusedPointerChanged(nullptr);
+                }
             }
         }
     );
@@ -437,17 +476,13 @@ void SeatInterface::Private::getKeyboard(wl_client *client, wl_resource *resourc
     keyboards << keyboard;
     if (keys.focus.surface && keys.focus.surface->client() == clientConnection) {
         // this is a keyboard for the currently focused keyboard surface
-        if (!keys.focus.keyboard) {
-            keys.focus.keyboard = keyboard;
-            keyboard->setFocusedSurface(keys.focus.surface, keys.focus.serial);
-        }
+        keys.focus.keyboards << keyboard;
+        keyboard->setFocusedSurface(keys.focus.surface, keys.focus.serial);
     }
     QObject::connect(keyboard, &QObject::destroyed, q,
         [keyboard,this] {
             keyboards.removeAt(keyboards.indexOf(keyboard));
-            if (keys.focus.keyboard == keyboard) {
-                keys.focus.keyboard = nullptr;
-            }
+            keys.focus.keyboards.removeOne(keyboard);
         }
     );
     emit q->keyboardCreated(keyboard);
@@ -614,19 +649,16 @@ void SeatInterface::setFocusedPointerSurface(SurfaceInterface *surface, const QM
         return;
     }
     const quint32 serial = d->display->nextSerial();
-    if (d->globalPointer.focus.pointer) {
-        d->globalPointer.focus.pointer->setFocusedSurface(nullptr, serial);
+    for (auto it = d->globalPointer.focus.pointers.constBegin(), end = d->globalPointer.focus.pointers.constEnd(); it != end; ++it) {
+        (*it)->setFocusedSurface(nullptr, serial);
     }
     if (d->globalPointer.focus.surface) {
         disconnect(d->globalPointer.focus.destroyConnection);
     }
     d->globalPointer.focus = Private::Pointer::Focus();
     d->globalPointer.focus.surface = surface;
-    PointerInterface *p = d->pointerForSurface(surface);
-    if (p && !p->resource()) {
-        p = nullptr;
-    }
-    d->globalPointer.focus.pointer = p;
+    auto p = d->pointersForSurface(surface);
+    d->globalPointer.focus.pointers = p;
     if (d->globalPointer.focus.surface) {
         d->globalPointer.focus.destroyConnection = connect(surface, &QObject::destroyed, this,
             [this] {
@@ -639,17 +671,24 @@ void SeatInterface::setFocusedPointerSurface(SurfaceInterface *surface, const QM
         d->globalPointer.focus.transformation = transformation;
         d->globalPointer.focus.serial = serial;
     }
-    emit focusedPointerChanged(p);
-    if (!p) {
+    if (p.isEmpty()) {
+        emit focusedPointerChanged(nullptr);
         return;
     }
-    p->setFocusedSurface(surface, serial);
+    // TODO: signal with all pointers
+    emit focusedPointerChanged(p.first());
+    for (auto it = p.constBegin(), end = p.constEnd(); it != end; ++it) {
+        (*it)->setFocusedSurface(surface, serial);
+    }
 }
 
 PointerInterface *SeatInterface::focusedPointer() const
 {
     Q_D();
-    return d->globalPointer.focus.pointer;
+    if (d->globalPointer.focus.pointers.isEmpty()) {
+        return nullptr;
+    }
+    return d->globalPointer.focus.pointers.first();
 }
 
 void SeatInterface::setFocusedPointerSurfacePosition(const QPointF &surfacePosition)
@@ -734,8 +773,10 @@ void SeatInterface::pointerAxis(Qt::Orientation orientation, quint32 delta)
         // ignore
         return;
     }
-    if (d->globalPointer.focus.pointer && d->globalPointer.focus.surface) {
-        d->globalPointer.focus.pointer->axis(orientation, delta);
+    if (d->globalPointer.focus.surface) {
+        for (auto it = d->globalPointer.focus.pointers.constBegin(), end = d->globalPointer.focus.pointers.constEnd(); it != end; ++it) {
+            (*it)->axis(orientation, delta);
+        }
     }
 }
 
@@ -758,11 +799,18 @@ void SeatInterface::pointerButtonPressed(quint32 button)
         // ignore
         return;
     }
-    if (d->globalPointer.focus.pointer && d->globalPointer.focus.surface) {
-        d->globalPointer.focus.pointer->buttonPressed(button, serial);
-        if (d->globalPointer.focus.surface == d->keys.focus.surface && d->keys.focus.keyboard) {
+    if (d->globalPointer.focus.surface) {
+        for (auto it = d->globalPointer.focus.pointers.constBegin(), end = d->globalPointer.focus.pointers.constEnd(); it != end; ++it) {
+            (*it)->buttonPressed(button, serial);
+        }
+        if (d->globalPointer.focus.surface == d->keys.focus.surface) {
             // update the focused child surface
-            d->keys.focus.keyboard->d_func()->focusChildSurface(d->globalPointer.focus.pointer->d_func()->focusedChildSurface, serial);
+            auto p = focusedPointer();
+            if (p) {
+                for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
+                    (*it)->d_func()->focusChildSurface(p->d_func()->focusedChildSurface, serial);
+                }
+            }
         }
     }
 }
@@ -791,8 +839,10 @@ void SeatInterface::pointerButtonReleased(quint32 button)
         d->endDrag(serial);
         return;
     }
-    if (d->globalPointer.focus.pointer && d->globalPointer.focus.surface) {
-        d->globalPointer.focus.pointer->buttonReleased(button, serial);
+    if (d->globalPointer.focus.surface) {
+        for (auto it = d->globalPointer.focus.pointers.constBegin(), end = d->globalPointer.focus.pointers.constEnd(); it != end; ++it) {
+            (*it)->buttonReleased(button, serial);
+        }
     }
 }
 
@@ -811,13 +861,27 @@ quint32 SeatInterface::pointerButtonSerial(quint32 button) const
     return it.value();
 }
 
+void SeatInterface::relativePointerMotion(const QSizeF &delta, const QSizeF &deltaNonAccelerated, quint64 microseconds)
+{
+    Q_D();
+    if (d->globalPointer.focus.surface) {
+        for (auto it = d->globalPointer.focus.pointers.constBegin(), end = d->globalPointer.focus.pointers.constEnd(); it != end; ++it) {
+            (*it)->relativeMotion(delta, deltaNonAccelerated, microseconds);
+        }
+    }
+}
+
 void SeatInterface::keyPressed(quint32 key)
 {
     Q_D();
     d->keys.lastStateSerial = d->display->nextSerial();
-    d->updateKey(key, Private::Keyboard::State::Pressed);
-    if (d->keys.focus.keyboard && d->keys.focus.surface) {
-        d->keys.focus.keyboard->keyPressed(key, d->keys.lastStateSerial);
+    if (!d->updateKey(key, Private::Keyboard::State::Pressed)) {
+        return;
+    }
+    if (d->keys.focus.surface) {
+        for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
+            (*it)->keyPressed(key, d->keys.lastStateSerial);
+        }
     }
 }
 
@@ -825,9 +889,13 @@ void SeatInterface::keyReleased(quint32 key)
 {
     Q_D();
     d->keys.lastStateSerial = d->display->nextSerial();
-    d->updateKey(key, Private::Keyboard::State::Released);
-    if (d->keys.focus.keyboard && d->keys.focus.surface) {
-        d->keys.focus.keyboard->keyReleased(key, d->keys.lastStateSerial);
+    if (!d->updateKey(key, Private::Keyboard::State::Released)) {
+        return;
+    }
+    if (d->keys.focus.surface) {
+        for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
+            (*it)->keyReleased(key, d->keys.lastStateSerial);
+        }
     }
 }
 
@@ -841,19 +909,15 @@ void SeatInterface::setFocusedKeyboardSurface(SurfaceInterface *surface)
 {
     Q_D();
     const quint32 serial = d->display->nextSerial();
-    if (d->keys.focus.keyboard) {
-        d->keys.focus.keyboard->setFocusedSurface(nullptr, serial);
+    for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
+        (*it)->setFocusedSurface(nullptr, serial);
     }
     if (d->keys.focus.surface) {
         disconnect(d->keys.focus.destroyConnection);
     }
     d->keys.focus = Private::Keyboard::Focus();
     d->keys.focus.surface = surface;
-    KeyboardInterface *k = d->keyboardForSurface(surface);
-    if (k && !k->resource()) {
-        k = nullptr;
-    }
-    d->keys.focus.keyboard = k;
+    d->keys.focus.keyboards = d->keyboardsForSurface(surface);
     if (d->keys.focus.surface) {
         d->keys.focus.destroyConnection = connect(surface, &QObject::destroyed, this,
             [this] {
@@ -865,15 +929,15 @@ void SeatInterface::setFocusedKeyboardSurface(SurfaceInterface *surface)
         // selection?
         d->keys.focus.selection = d->dataDeviceForSurface(surface);
         if (d->keys.focus.selection) {
-            if (d->currentSelection) {
+            if (d->currentSelection && d->currentSelection->selection()) {
                 d->keys.focus.selection->sendSelection(d->currentSelection);
             } else {
                 d->keys.focus.selection->sendClearSelection();
             }
         }
     }
-    if (k) {
-        k->setFocusedSurface(surface, serial);
+    for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
+        (*it)->setFocusedSurface(surface, serial);
     }
     // focused text input surface follows keyboard
     if (hasKeyboard()) {
@@ -910,8 +974,10 @@ void SeatInterface::updateKeyboardModifiers(quint32 depressed, quint32 latched,
     }
     const quint32 serial = d->display->nextSerial();
     d->keys.modifiers.serial = serial;
-    if (d->keys.focus.keyboard && d->keys.focus.surface) {
-        d->keys.focus.keyboard->updateModifiers(depressed, latched, locked, group, serial);
+    if (d->keys.focus.surface) {
+        for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
+            (*it)->updateModifiers(depressed, latched, locked, group, serial);
+        }
     }
 }
 
@@ -1000,7 +1066,10 @@ QVector< quint32 > SeatInterface::pressedKeys() const
 KeyboardInterface *SeatInterface::focusedKeyboard() const
 {
     Q_D();
-    return d->keys.focus.keyboard;
+    if (d->keys.focus.keyboards.isEmpty()) {
+        return nullptr;
+    }
+    return d->keys.focus.keyboards.first();
 }
 
 void SeatInterface::cancelTouchSequence()
@@ -1083,18 +1152,20 @@ qint32 SeatInterface::touchDown(const QPointF &globalPosition)
         d->touchInterface.focus.touch->down(id, serial, globalPosition - d->touchInterface.focus.offset);
     } else if (id == 0 && focusedTouchSurface()) {
 #if HAVE_LINUX_INPUT_H
-        auto p = d->pointerForSurface(focusedTouchSurface());
-        if (!p) {
+        auto p = d->pointersForSurface(focusedTouchSurface());
+        if (p.isEmpty()) {
             return id;
         }
         const QPointF pos = globalPosition - d->touchInterface.focus.offset;
-        wl_pointer_send_enter(p->resource(), serial,
-                          focusedTouchSurface()->resource(),
-                          wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
-        wl_pointer_send_motion(p->resource(), timestamp(),
-                                wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
+        for (auto it = p.constBegin(), end = p.constEnd(); it != end; ++it) {
+            wl_pointer_send_enter((*it)->resource(), serial,
+                            focusedTouchSurface()->resource(),
+                            wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
+            wl_pointer_send_motion((*it)->resource(), timestamp(),
+                                    wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
 
-        wl_pointer_send_button(p->resource(), serial, timestamp(), BTN_LEFT, WL_POINTER_BUTTON_STATE_PRESSED);
+            wl_pointer_send_button((*it)->resource(), serial, timestamp(), BTN_LEFT, WL_POINTER_BUTTON_STATE_PRESSED);
+        }
 #endif
     }
 
@@ -1108,14 +1179,16 @@ void SeatInterface::touchMove(qint32 id, const QPointF &globalPosition)
     if (d->touchInterface.focus.touch && d->touchInterface.focus.surface) {
         d->touchInterface.focus.touch->move(id, globalPosition - d->touchInterface.focus.offset);
     } else if (id == 0 && focusedTouchSurface()) {
-        auto p = d->pointerForSurface(focusedTouchSurface());
-        if (!p) {
+        auto p = d->pointersForSurface(focusedTouchSurface());
+        if (p.isEmpty()) {
             return;
         }
 
         const QPointF pos = globalPosition - d->touchInterface.focus.offset;
-        wl_pointer_send_motion(p->resource(), timestamp(),
-                                wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
+        for (auto it = p.constBegin(), end = p.constEnd(); it != end; ++it) {
+            wl_pointer_send_motion((*it)->resource(), timestamp(),
+                                   wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
+        }
     } 
 }
 
@@ -1128,12 +1201,14 @@ void SeatInterface::touchUp(qint32 id)
     } else if (id == 0 && focusedTouchSurface()) {
 #if HAVE_LINUX_INPUT_H
         const quint32 serial = display()->nextSerial();
-        auto p = d->pointerForSurface(focusedTouchSurface());
-        if (!p) {
+        auto p = d->pointersForSurface(focusedTouchSurface());
+        if (p.isEmpty()) {
             return;
         }
 
-        wl_pointer_send_button(p->resource(), serial, timestamp(), BTN_LEFT, WL_POINTER_BUTTON_STATE_RELEASED);
+        for (auto it = p.constBegin(), end = p.constEnd(); it != end; ++it) {
+            wl_pointer_send_button((*it)->resource(), serial, timestamp(), BTN_LEFT, WL_POINTER_BUTTON_STATE_RELEASED);
+        }
 #endif
     }
     d->touchInterface.ids.removeAll(id);
@@ -1265,9 +1340,11 @@ void SeatInterface::setSelection(DataDeviceInterface *dataDevice)
     if (d->currentSelection == dataDevice) {
         return;
     }
+    // cancel the previous selection
+    d->cancelPreviousSelection(dataDevice);
     d->currentSelection = dataDevice;
     if (d->keys.focus.selection) {
-        if (dataDevice) {
+        if (dataDevice && dataDevice->selection()) {
             d->keys.focus.selection->sendSelection(dataDevice);
         } else {
             d->keys.focus.selection->sendClearSelection();
