@@ -152,6 +152,77 @@ public:
      **/
     QPointer<ContrastInterface> contrast() const;
 
+    /**
+     * Whether the SurfaceInterface is currently considered to be mapped.
+     * A SurfaceInterface is mapped if it has a non-null BufferInterface attached.
+     * If the SurfaceInterface references a SubSurfaceInterface it is only considered
+     * mapped if it has a BufferInterface attached and the parent SurfaceInterface is mapped.
+     *
+     * @returns Whether the SurfaceInterface is currently mapped
+     * @since 5.22
+     **/
+    bool isMapped() const;
+
+    /**
+     * Returns the tracked damage since the last call to @link resetTrackedDamage.
+     * In contrast to @link damage this method does not reset the damage when
+     * a new BufferInterface gets committed. This allows a compositor to properly
+     * track the damage over multiple commits even if it didn't render each new
+     * BufferInterface.
+     *
+     * The damage gets reset whenever @link resetTrackedDamage is called.
+     * This allows a compositor to properly track the change in its rendering scene
+     * for this SurfaceInterface. After it updates its internal state (e.g. by creating
+     * an OpenGL texture from the BufferInterface) it can invoke @link resetTrackedDamage
+     * and the damage tracker will start to track further damage changes.
+     *
+     * @returns Combined damage since last call to resetTrackedDamage
+     * @see damage
+     * @see resetTrackedDamage
+     * @since 5.22
+     **/
+    QRegion trackedDamage() const;
+
+    /**
+     * Reset the damage tracking. The compositor should invoke this method once it updated
+     * it's internal state and processed the current damage.
+     * @see trackedDamage
+     * @since 5.22
+     **/
+    void resetTrackedDamage();
+
+    /**
+     * Finds the SurfaceInterface at the given @p position in surface-local coordinates.
+     * This can be either a descendant SurfaceInterface honoring the stacking order or
+     * the SurfaceInterface itself if its geometry contains the given @p position.
+     *
+     * If no such SurfaceInterface is found, e.g. because the SurfaceInterface is unmapped,
+     * @c nullptr is returned.
+     *
+     * @param position The position in surface-local coordinates
+     * @returns Child surface at the given @p position or surface itself at the position, might be @c nullptr
+     * @since 5.22
+     **/
+    SurfaceInterface *surfaceAt(const QPointF &position);
+
+    /**
+     * Sets the @p outputs this SurfaceInterface overlaps with, may be empty.
+     *
+     * The compositor should update whenever the SurfaceInterface becomes visible on
+     * an OutputInterface by e.g. getting (un)mapped, resized, moved, etc.
+     *
+     * @see outputs
+     * @since 5.27
+     **/
+    void setOutputs(const QVector<OutputInterface *> &outputs);
+
+    /**
+     * @returns All OutputInterfaces the SurfaceInterface is on.
+     * @see setOutputs
+     * @since 5.27
+     **/
+    QVector<OutputInterface *> outputs() const;
+
     /**
      * @returns The SurfaceInterface for the @p native resource.
      **/
@@ -200,6 +271,11 @@ Q_SIGNALS:
      * @since 5.5
      **/
     void contrastChanged();
+    /**
+     * Emitted whenever the tree of sub-surfaces changes in a way which requires a repaint.
+     * @since 5.22
+     **/
+    void subSurfaceTreeChanged();
 
 private:
     friend class CompositorInterface;
@@ -217,4 +293,6 @@ private:
 }
 }
 
+Q_DECLARE_METATYPE(KWayland::Server::SurfaceInterface*)
+
 #endif
